@@ -109,14 +109,12 @@ class TypeCheckedTree(TestCase):
 		try:			
 			preorder(Tree(5, Tree(6), Tree(7.0)))
 		except TypeCheckError, e:
-			assert isinstance(e.internal, _TC_IndexError)
-			assert e.internal.index == 0
-			assert isinstance(e.internal.inner, _TC_TreeChildTypeError)
-			assert e.internal.inner.child == 'right'
-			assert isinstance(e.internal.inner.inner, _TC_TypeError)
-			assert e.internal.inner.inner.right == int
-			assert e.internal.inner.inner.wrong == float
-			self.assertEqual(str(e), "in (Tree(5, Tree(6), Tree(7.0)),), at index 0, in the right child, expected <type 'int'>, got <type 'float'>")
+			assert isinstance(e.internal, _TC_TreeChildTypeError)
+			assert e.internal.child == 'right'
+			assert isinstance(e.internal.inner, _TC_TypeError)
+			assert e.internal.inner.right == int
+			assert e.internal.inner.wrong == float
+			self.assertEqual(str(e), "Argument tree: for Tree(5, Tree(6), Tree(7.0)), in the right child, expected <type 'int'>, got <type 'float'>")
 		else:
 			raise AssertionError("Succeeded incorrectly")
 
