@@ -715,11 +715,9 @@ class ExtensibleSigTests(TestCase):
 		try:
 			foo('a', 5)
 		except TypeCheckError, e:
-			assert isinstance(e.internal, _TC_IndexError)
-			assert e.internal.index == 0
-			assert isinstance(e.internal.inner, _TC_TypeError)
-			assert e.internal.inner.wrong == str
-			assert e.internal.inner.right == 5
+			assert isinstance(e.internal, _TC_TypeError)
+			assert e.internal.wrong == str
+			assert e.internal.right == 5
 		else:
 			raise AssertionError("Succeeded incorrectly")
 	
@@ -764,13 +762,149 @@ class ExtensibleSigTests(TestCase):
 		try:
 			foo('a', 5)
 		except TypeCheckError, e:
-			assert isinstance(e.internal, _TC_IndexError)
-			assert e.internal.index == 0
-			assert isinstance(e.internal.inner, _TC_TypeError)
-			assert e.internal.inner.wrong == str
-			assert e.internal.inner.right == 5
+			assert isinstance(e.internal, _TC_TypeError)
+			assert e.internal.wrong == str
+			assert e.internal.right == 5
 		else:
 			raise AssertionError("Succeeded incorrectly")
+
+class MyError(Exception):
+	pass
+			
+class TestUsingFunctionsInSigs(TestCase):
+	def setUp(self):
+		def returns(val):
+			def _returns(obj):
+				return val
+			return _returns
+		self.returns = returns
+	
+		def raises(exc):
+			def _raises(obj):
+				raise exc
+			return _raises
+		self.raises = raises
+
+	def test_function_returns_True(self):
+		from typecheck import typecheck_args
+		
+		@typecheck_args(self.returns(True))
+		def bar(a):
+			return a
+			
+		assert bar(7) == 7
+		
+	def test_function_returns_None(self):
+		from typecheck import typecheck_args
+		
+		@typecheck_args(self.returns(None))
+		def bar(a):
+			return a
+			
+		assert bar(7) == 7
+		
+	def test_function_returns_False(self):
+		from typecheck import typecheck_args, TypeCheckError
+		from typecheck import _TC_FunctionError
+		
+		checking_func = self.returns(False)
+		
+		@typecheck_args(checking_func)
+		def bar(a):
+			return a
+			
+		try:
+			assert bar(7) == 7
+		except TypeCheckError, e:
+			assert isinstance(e.internal, _TC_FunctionError)
+			assert e.internal.rejected_obj == 7
+			self.assertEqual(str(e), "Argument a: 7 was rejected by %s" % checking_func)
+		else:
+			raise AssertionError("Failed to raise TypeCheckError")
+			
+	def test_function_raises_tc_exception(self):
+		from typecheck import typecheck_args, TypeCheckError
+		from typecheck import _TC_LengthError
+		
+		@typecheck_args(self.raises(_TC_LengthError(5, 6)))
+		def bar(a):
+			return a
+			
+		try:
+			assert bar(7) == 7
+		except TypeCheckError, e:
+			assert isinstance(e.internal, _TC_LengthError)
+			assert e.internal.wrong == 5
+			assert e.internal.right == 6
+			self.assertEqual(str(e), "Argument a: for 7, length was 5, expected 6")
+		else:
+			raise AssertionError("Failed to raise TypeCheckError")
+			
+	def test_function_raises_other_exception(self):
+		from typecheck import typecheck_args
+		
+		@typecheck_args(self.raises(MyError()))
+		def bar(a):
+			return a
+			
+		try:
+			assert bar(7) == 7
+		except MyError:
+			pass
+		else:
+			raise AssertionError("Failed to raise MyError")
+
+class TestUsingClassmethodsInSigs(TestUsingFunctionsInSigs):			
+	def setUp(self):
+		def returns(val):
+			class Foo(object):
+				@classmethod
+				def _returns(cls, obj):
+					return val
+			return Foo._returns
+		self.returns = returns
+	
+		def raises(exc):
+			class Foo(object):
+				@classmethod
+				def _raises(cls, obj):
+					raise exc
+			return Foo._raises
+		self.raises = raises
+
+class TestUsingStaticmethodsInSigs(TestUsingFunctionsInSigs):			
+	def setUp(self):
+		def returns(val):
+			class Foo(object):
+				@staticmethod
+				def _returns(obj):
+					return val
+			return Foo._returns
+		self.returns = returns
+	
+		def raises(exc):
+			class Foo(object):
+				@staticmethod
+				def _raises(obj):
+					raise exc
+			return Foo._raises
+		self.raises = raises
+		
+class TestUsingInstancemethodsInSigs(TestUsingFunctionsInSigs):			
+	def setUp(self):
+		def returns(val):
+			class Foo(object):
+				def _returns(self, obj):
+					return val
+			return Foo()._returns
+		self.returns = returns
+	
+		def raises(exc):
+			class Foo(object):
+				def _raises(self, obj):
+					raise exc
+			return Foo()._raises
+		self.raises = raises
 			
 ### Bookkeeping ###
 if __name__ == '__main__':
