@@ -1,7 +1,7 @@
 import inspect
 import types
 
-from types import GeneratorType, FunctionType, MethodType
+from types import GeneratorType, FunctionType, MethodType, ClassType, TypeType
 
 # Controls whether typechecking is on (True) or off (False)
 enable_checking = True
@@ -1036,6 +1036,95 @@ class Class(CheckType):
 		
 		if not isinstance(to_check, self.class_obj):
 			raise _TC_TypeError(to_check, self.class_obj)
+            
+class Typeclass(CheckType):
+    bad_members = dict.fromkeys(['__class__', '__new__', '__init__'], True)
+
+    def __init__(self, *types):
+        if len(types) == 0:
+            raise TypeError("Must supply at least one type to __init__()")
+    
+        self._cache = set()
+        self._interface = set()
+        self._instances = set()
+        for t in types:
+            self.add_instance(t)
+                
+        self._calculate_interface()
+        
+    def recalculate_interface(self):
+        self._cache = self._instances.copy()
+        self._calculate_interface()
+                
+    def instances(self):
+        return list(self._instances)
+        
+    def interface(self):
+        return list(self._interface)
+        
+    def has_instance(self, instance):
+        return instance in self._instances
+        
+    def add_instance(self, instance):
+        if isinstance(instance, self.__class__):
+            for inst in instance.instances():
+                self._instances.add(inst)
+                self._cache.add(inst)
+        elif isinstance(instance, (ClassType, TypeType)):
+            self._instances.add(instance)
+            self._cache.add(instance)
+        else:
+            raise TypeError("All instances must be classes or types")
+        
+    def intersect(self, other):
+        if isinstance(other, self.__class__):
+            new_instances = other.instances()
+        else:
+            new_instances = other
+            
+        self._instances.update(new_instances)
+        self._cache.update(new_instances)
+        self._calculate_interface()
+        
+    def _calculate_interface(self):
+        bad_members = self.bad_members
+    
+        for instance in self._instances:
+            inst_attrs = []
+        
+            for attr, obj in instance.__dict__.items():
+                if callable(obj) and attr not in bad_members:
+                    inst_attrs.append(attr)
+            
+            if len(self._interface) == 0:
+                self._interface = set(inst_attrs)
+            else:
+                self._interface.intersection_update(inst_attrs)
+                
+    def __typecheck__(self, func, to_check):
+        if to_check.__class__ in self._cache:
+            return
+            
+        for method in self._interface:
+            if not hasattr(to_check, method):
+                raise _TC_MissingAttrError(method)
+
+            attr = getattr(to_check, method)
+            if not callable(attr):
+                raise _TC_AttrError(method, _TC_TypeError(attr, IsCallable()))
+                
+        self._cache.add(to_check.__class__)
+        
+    def __eq__(self, other):
+        if self.__class__ is not other.__class__:
+            return False
+        return self._instances == other._instances
+        
+    def __hash__(self):
+        return hash(str(self.__class__) + str(hash(frozenset(self._instances))))
+        
+    def __call__(self):
+        return self
 
 # The current implementation of Self relies on the TypeVariables machinery
 _Self = TypeVariables("this is the class of the invocant")
